{*------------------------------------------------------------------------------
 Unit de gestion des mthodes de reconnaissance de carctres.

 @Author    Thierry LABORDE
-------------------------------------------------------------------------------}
unit UGestionOCR;

interface

uses
  Math, SysUtils;


function CalculAngle(x1,y1,x2,y2:double):double;
function DistanceEntreCourbe(Courbe1,Courbe2:string;i,j:integer):integer;
function DistanceDeLevenshtein(Courbe1,Courbe2:String):integer;


implementation


{*------------------------------------------------------------------------------
  Fonction pour calculer un angle en 2 points sur un canvas

  @param    X1 : Coordonnes X du 1er point
  @param    Y1 : Coordonnes Y du 1er point
  @param    X2 : Coordonnes X du 2me point
  @param    Y2 : Coordonnes Y du 2me point
  @Return   Renvoi l'angle en degr
-------------------------------------------------------------------------------}
function CalculAngle(x1,y1,x2,y2:double):double;
var
  a,ang,x,y:double;
begin
  Ang:=0;
  x:=x2-x1;
  y:=y1-y2;
  if x=0 then
  begin
   if y>0 then
     ang:=Pi/2
   else
     if y<0 then
       ang:=3*Pi/2
     else
       ang:=0;
  end
  else
  begin
    a:=arctan(y/x);
    if x>0 then
      if y>=0 then
        ang:=a
      else
        ang:=a+2*Pi;
    if x<0 then ang:=a+pi;
  end;
  Ang:=RadToDeg(Ang);
  result:=ang;
end;

{*------------------------------------------------------------------------------
  Fonction pour calculer la distance entre 2 points de 2 Courbes

  @param    Courbe1  String de la 1re courbe
  @param    Courbe2  String de la 2me courbe
  @Param    i        Numro du point de la 1re Courbe
  @Param    j        Numro du point de la 2me Courbe
  @return   Renvoi la distance entre les 2 points des 2 courbes
-------------------------------------------------------------------------------}
function DistanceEntreCourbe(Courbe1,Courbe2:string;i,j:integer):integer;
var
  a,b,c:integer;
begin
  a:=StrToInt(Courbe1[i]);
  b:=StrToInt(Courbe2[j]);
  c:=abs(a-b);
  if c>4 then c:=8-c;
  result:=c;
end;

{*------------------------------------------------------------------------------
  Fonction pour calculer de Levenshtein entre 2 courbes

  @param    Courbe1  String de la 1re courbe
  @param    Courbe2  String de la 2me courbe
  @return   Renvoi la distance de Levenshtein entre les 2 courbes
-------------------------------------------------------------------------------}
function DistanceDeLevenshtein(Courbe1,Courbe2:String):integer;
var
  d  : array of array of integer;
  DTW: array of array of integer;
  i,j,n,m,cout : integer;
begin
  n:=Length(Courbe1);
  m:=Length(Courbe2);
  SetLength(d,n+1,m+1);
  for i:=1 to n do
    for j:=1 to m do
      d[i,j]:=DistanceEntreCourbe(Courbe1,Courbe2,i,j);
  SetLength(DTW,n+1,m+1);
  for i:=1 to m do
    DTW[0,i]:=100000;
  for i:=1 to n do
    DTW[i,0]:=100000;
  DTW[0,0]:=0;
  for i:=1 to n do
  begin
    for j:=1 to m do
    begin
      cout:=d[i,j];
      DTW[i,j]:=Min(Min(DTW[i-1,j]+cout, DTW[i,j-1]+cout),DTW[i-1,j-1]+cout);
    end;
  end;
  result:=DTW[n,m];
end;


end.
