{*------------------------------------------------------------------------------
 Fentre principale de l'application PDA OCR.

 @Author    Thierry LABORDE
-------------------------------------------------------------------------------}
unit UMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Math, StrUtils;

Const
  COLOR_PAINTBOX = $006E6E00;   // Couleur de la Zone de dessin
  CURSEUR_STYLO  = 1;           // Numro du curseur du stylo

type
  TFrmMain = class(TForm)
    PbDessinMaj: TPaintBox;
    MmText: TMemo;
    ImgFond: TImage;
    ImgFermer: TImage;
    ImgReduire: TImage;
    PbDessinMin: TPaintBox;
    ShSeparation: TShape;
    procedure PbDessinMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PbDessinMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure PbDessinMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure ImgFermerClick(Sender: TObject);
    procedure ImgReduireClick(Sender: TObject);
  private
    { Dclarations prives }
    CaractereDessin:String;
    AngleRef : array[1..8] of Double;
    Procedure Initialisation;
    procedure WMNCHitTest(var msg: TWMNCHitTest); message WM_NCHITTEST;
  public
    { Dclarations publiques }
  end;

var
  FrmMain: TFrmMain;

implementation

{$R *.dfm}
{$R Curseur.Res}

uses
  UGestionOcr, UgestionSkin, UGestionDico;


{*------------------------------------------------------------------------------
  Procdure dclenche lors de l'appui sur le bouton de la souris sur la zone
  de dessin pour initialiser la position de dpart du dessin
-------------------------------------------------------------------------------}
procedure TFrmMain.PbDessinMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  (Sender As TPaintBox).Canvas.MoveTo(X,Y);
  // Si on commence un dessin sur une des PaintBox on cache l'autre tant qu'on
  // a pas fini le dessin
  if (Sender As TPaintBox).Name='PbDessinMin' then
    PbDessinMaj.Visible:=False
  else
    PbDessinMin.Visible:=False;
end;

{*------------------------------------------------------------------------------
  Procdure dclenche lors du dessin du caractre pour enregistrer les angles
  et dessiner le caractre
-------------------------------------------------------------------------------}
procedure TFrmMain.PbDessinMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  i,index:integer;
  Distance,Ang:extended;
begin
  if Shift=[ssLeft] then
  begin
    // On ne refait le calcul que tous les 10 pixels de dplacement pour viter
    // d'avoir trop d'erreurs de reconnaissance
    if (Abs(X-(Sender As TPaintBox).Canvas.PenPos.X)>10) or (Abs(Y-(Sender As TPaintBox).Canvas.PenPos.Y)>10) then
    begin
      Index:=0;
      Ang:=CalculAngle((Sender As TPaintBox).Canvas.PenPos.X,(Sender As TPaintBox).Canvas.PenPos.Y,X,Y);
      (Sender As TPaintBox).Canvas.LineTo(X,Y);
      Distance:=100000;
      for i:=1 to High(AngleRef) do
      begin
        if (Distance>sqrt(sqr(Ang-AngleRef[i]))) then
        begin
          Distance:=sqrt(sqr(Ang-AngleRef[i]));
          Index:=i;
        end;
      end;
      if (Index<>0) and (rightStr(CaractereDessin,1)<>inttostr(Index)) then
        CaractereDessin:=CaractereDessin+inttostr(Index);
    end;
  end;
end;

{*------------------------------------------------------------------------------
  Procdure dclenche lors de la cration de la fentre pour faire
  l'initialisation
-------------------------------------------------------------------------------}
procedure TFrmMain.FormCreate(Sender: TObject);
begin
  Initialisation;
end;

{*------------------------------------------------------------------------------
  Procdure dclenche lors du relachement du bouton de la souris sur la zone de
  dessin pour lancer la reconnaissance du carctre
-------------------------------------------------------------------------------}
procedure TFrmMain.PbDessinMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  i,index:integer;
  Distance:extended;
begin
  Index:=-1;
  Distance:=100000;
  for i:=0 to ListeDico.Count-1 do
  begin
    if (Distance>DistanceDeLevenshtein(CaractereDessin,Tdico(ListeDico[i]).Dessin)) then
    begin
      Distance:=DistanceDeLevenshtein(CaractereDessin,Tdico(ListeDico[i]).Dessin);
      Index:=i;
    end;
  end;
  if index<>-1 then
  begin
    if Length(Tdico(ListeDico[Index]).Lettre)>1 then
    begin
      if strtoint(Tdico(ListeDico[Index]).Lettre)=99 then  // cas de la commande pour sortir de l'application
        ImgFermer.OnClick(Sender)
      else
        keybd_event(strtoint(Tdico(ListeDico[Index]).Lettre),0,WM_KEYDOWN,0);
    end
    else
    begin
      if (Sender As TPaintBox).Name='PbDessinMaj' then
        MmText.Text:=MmText.Text+UpperCase(Tdico(ListeDico[Index]).Lettre)
      else
        MmText.Text:=MmText.Text+LowerCase(Tdico(ListeDico[Index]).Lettre);
    end;
    MmText.SelStart:=Length(MmText.Text);
  end;
  CaractereDessin:='';
  (Sender As TPaintBox).Canvas.FillRect((Sender As TPaintBox).Canvas.ClipRect);
  // le dessin est fini on raffiche les 2 PaintBox
  PbDessinMaj.Visible:=True;
  PbDessinMin.Visible:=True;
end;

{*------------------------------------------------------------------------------
  Procdure dclenche lors de l'affichage de la fentre pour skiner la fentre
   partir de l'image de fond.
-------------------------------------------------------------------------------}
procedure TFrmMain.FormShow(Sender: TObject);
begin
  SkinFenetre(ImgFond.Picture.Bitmap);
end;

{*------------------------------------------------------------------------------
  Procdure pour pouvoir dplacer la fentre lorsque l'on se trouve sur le haut
  de la fentre et qu'on clique en bougeant la souris.
-------------------------------------------------------------------------------}
procedure TFrmMain.WMNCHitTest(var msg: TWMNCHittest);
var
  pt: TPoint;
begin
  inherited;
  pt:= ScreenToClient(Point(msg.XPos, msg.YPos));
  if PtInRect(Rect(0, 0, ClientWidth, 60), pt) then
    msg.Result:= HTCAPTION;
end;

{*------------------------------------------------------------------------------
  Procdure d'initialisation des paramtres
-------------------------------------------------------------------------------}
Procedure TFrmMain.Initialisation;
begin
  Try
    ImgFond.Picture.LoadFromFile('Fond.bmp');
  except
    Showmessage('Problme lors du chargement de l''image de fond, vrifez votre installation !!!');
    Application.terminate;
    exit;
  end;
  DoubleBuffered:=True;
  CaractereDessin:='';
  PbDessinMaj.Canvas.Pen.Width:=2;
  PbDessinMin.Canvas.Pen.Width:=2;
  FrmMain.Color:=COLOR_PAINTBOX;
  Screen.Cursors[CURSEUR_STYLO] := LoadCursor(HInstance, 'STYLO');
  PbDessinMaj.Cursor := CURSEUR_STYLO;
  PbDessinMin.Cursor := CURSEUR_STYLO;
  ListeDico:=TList.Create;
  // dfinitions des angles de rfrences
  AngleRef[1]:=90;
  AngleRef[2]:=45;
  AngleRef[3]:=0;
  AngleRef[4]:=315;
  AngleRef[5]:=270;
  AngleRef[6]:=225;
  AngleRef[7]:=180;
  AngleRef[8]:=135;
  if Not(ChargeDico) then Application.Terminate;
end;

{*------------------------------------------------------------------------------
  Procdure lors du click sur le bouton pour fermer l'application
-------------------------------------------------------------------------------}
procedure TFrmMain.ImgFermerClick(Sender: TObject);
begin
  if MessageDlg('Voulez vous quitter l''application ?',mtConfirmation,[Mbyes,MbNo],0)=MrYes then
  begin
    ListeDico.Free;
    Application.Terminate;
  end;
end;

{*------------------------------------------------------------------------------
  Procdure lors du click sur le bouton pour rduire l'application
-------------------------------------------------------------------------------}
procedure TFrmMain.ImgReduireClick(Sender: TObject);
begin
  Application.Minimize;
end;

end.
